@@ -998,7 +998,8 @@ class OssapiAsync:
         *,
         mode: Optional[GameModeT] = None,
         mods: Optional[ModT] = None,
-        type: Optional[RankingTypeT] = None
+        type: Optional[RankingTypeT] = None,
+        limit: Optional[int] = None
     ) -> BeatmapScores:
         """
         Get the top scores of a beatmap.
@@ -1013,13 +1014,16 @@ class OssapiAsync:
             Get the top scores set with exactly these mods, if passed.
         type
             How to order the scores. Defaults to ordering by score.
+        limit
+            How many results to return. Defaults to 50. Must be between 1 and
+            100.
 
         Notes
         -----
         Implements the `Get Beatmap Scores
         <https://osu.ppy.sh/docs/index.html#get-beatmap-scores>`__ endpoint.
         """
-        params = {"mode": mode, "mods": mods, "type": type}
+        params = {"mode": mode, "mods": mods, "type": type, "limit": limit}
         return await self._get(BeatmapScores, f"/beatmaps/{beatmap_id}/scores",
             params)
 
@@ -1045,7 +1049,7 @@ class OssapiAsync:
         Notes
         -----
         Combines the
-        `Get Beatmap <https://osu.ppy.sh/docs/index.html#get-beatmapL>`_ and
+        `Get Beatmap <https://osu.ppy.sh/docs/index.html#get-beatmap>`_ and
         `Lookup Beatmap <https://osu.ppy.sh/docs/index.html#lookup-beatmap>`_
         endpoints.
         """
@@ -2180,24 +2184,35 @@ class OssapiAsync:
         <https://osu.ppy.sh/docs/index.html#scoresmodescoredownload>`__
         endpoint.
         """
+        from aiohttp import ClientSession, ContentTypeError
+
         url = f"{self.BASE_URL}/scores/{mode.value}/{score_id}/download"
-        r = await self.session.request_async("GET", url)
+
+        aiohttp_session = ClientSession()
+        r = await self.session.request_async("GET", url,
+            session=aiohttp_session)
 
         # if the response above succeeded, it will return a raw string
         # instead of json. If it didn't succeed, it will return json with an
         # error.
-        # So always try parsing as json to check if there's an error. If parsin
+        # So always try parsing as json to check if there's an error. If parsing
         # fails, just assume the request succeeded and move on.
+        # TODO we probably want to be checking headers here instead.
+        # Should be x-osu-replay for valid response.
         try:
-            json_ = r.json()
+            json_ = await r.json()
+            await aiohttp_session.close()
             self._check_response(json_, url)
-        except json.JSONDecodeError:
+        except ContentTypeError:
             pass
 
-        if raw:
-            return r.content
+        content = await r.read()
+        await aiohttp_session.close()
 
-        replay = osrparse.Replay.from_string(r.content)
+        if raw:
+            return content
+
+        replay = osrparse.Replay.from_string(content)
         return Replay(replay, self)
 
 
