@@ -72,6 +72,23 @@ class _Model:
         """
         return None
 
+    @classmethod
+    def preprocess_data(cls, data):
+        """
+        A hook that allows model classes to modify data from the api before it
+        is used to instantiate the model.
+
+        For example, if a model attribute come as either a bool or an integer
+        (0 for false and 1 for true), this method can be overridden to convert
+        the integer to a boolean before model instantiation. This lets you
+        define the attribute as type bool instead of type Union[int, bool].
+
+        ``preprocess_data`` is called before ``override_class`` and
+        ``override_types``, so changes to the data in ``preprocess_data`` will
+        affect the data passed to those methods.
+        """
+        return data
+
 class ModelMeta(type):
     def __new__(cls, name, bases, dct):
         model = super().__new__(cls, name, bases, dct)
@@ -173,6 +190,9 @@ class Datetime(datetime, BaseModel):
             return datetime.strptime(value, "%Y-%m-%dT%H:%M:%S%z")
         if cls._matches_datetime(value, "%Y-%m-%d"):
             return datetime.strptime(value, "%Y-%m-%d")
+        # returned by eg https://osu.ppy.sh/api/v2/rooms/257524
+        if cls._matches_datetime(value, "%Y-%m-%d %H:%M:%S"):
+            return datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
         raise ValueError(f"invalid datetime string {value}")
 
     @staticmethod
